library(tidyr) #nettoie
library(dplyr) #déploie
library(lubridate) #gestion des dates
library(ggplot2) #graphique
library(scales) #pour ggplot avec gestion des écjelles numériques
library(stringr) #gestion des chaines de caractères
library(wordcloud2) #nuage de mots
library(tidytext) #nettoie le texte
library(lexicon) #accès à des lexiques sentiments
library(textdata) #accès à des fonctions d'analyse textuelles
library(broom) #test des différences loi de poisson

# library(purrr)
# library(twitteR)
# 
# 


#d'après http://varianceexplained.org/r/trump-tweets/
# David Robinson
#les commentaires en anglais sont de lui

#pour éviter d'aller chercher les tweets
load(url("http://varianceexplained.org/files/trump_tweets_df.rda"))

#1398 tweets entre le  "2015-12-14 20:09:15 UTC"   et le 2016-08-08 15:20:44 UTC"
#élections on eu lieu le 8 novembre 2016 et il s'était déclaré candidat en aout 2015



#Exploration des contextes d'envoi des tweets: suorces et heures


tweets <- trump_tweets_df %>%
  select(id, statusSource, text, created) %>%
  extract(statusSource, "source", "Twitter for (.*?)<") %>% #par defaut remove=TRUE
  filter(source %in% c("iPhone", "Android"))
tweets<-tweets %>% 
  mutate(hour = hour(with_tz(created, "EST"))) #A vérif-> question de zone

#Quelles heures et quelle sources ?
#cela peut donner une indication sur l'emetteur
tweets_heures<-tweets %>%
  count(source, hour = hour(with_tz(created, "EST"))) %>%  
  mutate(percent = n / sum(n)) 

ggplot(tweets_heures,aes(hour, percent, color = source)) +
  geom_line() +
  scale_y_continuous(labels = percent_format()) +  #lib scales
  labs(x = "Hour of day (EST)",
       y = "% of tweets",
       color = "")

# Trump on the Android does a lot more tweeting in the morning, while the campaign posts from the iPhone more 
# in the afternoon and early evening.
# 
# Another place we can spot a difference is in Trump’s anachronistic behavior of “manually retweeting” people 
# by copy-pasting their tweets, then surrounding them with quotation marks:
#   
# Almost all of these quoted tweets are posted from the Android:
#   

#2- Analyse de l'usage de tweeter - expert /non expert
#manière de retweeter, liens Hashtag, picture ou non

#2.1 - Hypothèse- quand le retweet est copié/collé il contient des guillemets=> opération de "débutant"
tweet_guill_counts <- tweets %>%
  count(source, retweetgood = ifelse(str_detect(text, '^"'), "manual", "expert")) #count number of tweets with/without quote marks
ggplot(tweet_guill_counts, aes(source, n, fill = retweetgood)) + #create plot
  geom_bar(stat = "identity", position = "dodge") + #add bar graph
  labs(x = "", y = "Number of tweets", fill = "Retweet") #label

#2.2 les hahstag
tweet_hash_counts <- tweets %>%
  filter(!str_detect(text, '^"')) %>% #remove text in quote marks
  count(source, picture = ifelse(str_detect(text, "#"), "Hashtag", "No Hashtag")) #count number of tweets with/without hashtags

ggplot(tweet_hash_counts, aes(source, n, fill = picture)) + #create plot
  geom_bar(stat = "identity", position = "dodge") + #add bar graph
  labs(x = "", y = "Number of tweets", fill = "") #label


#2.3- les illustrations par des photos
tweet_picture_counts <- tweets %>%
  filter(!str_detect(text, '^"')) %>%
  count(source,
        picture = ifelse(str_detect(text, "t.co"),
                         "Picture/link", "No picture/link"))

ggplot(tweet_picture_counts, aes(source, n, fill = picture)) +
  geom_bar(stat = "identity", position = "dodge") +
  labs(x = "", y = "Number of tweets", fill = "")


#3 - Analyse des contenus
#Comparison of words
# 
# Now that we’re sure there’s a difference between these two accounts, what can we say about the difference 
#in the content? We’ll use the tidytext package that Julia Silge and I developed.
# 
# We start by dividing into individual words using the unnest_tokens function (see this vignette for more), and removing some common “stopwords”2:

library(tidytext) #https://www.tidytextmining.com/tidytext.html
library(stopwords)

reg <- "([^A-Za-z\\d#@']|'(?![A-Za-z\\d#@]))"  #il faut comprendre cela!!
#extraction des mots - 
tweet_words <- tweets %>%
  filter(!str_detect(text, '^"')) %>%
  mutate(text2 = str_replace_all(text, "https://t.co/[A-Za-z\\d]+|&amp;", "")) 
tweet_words <- tweet_words %>%
  unnest_tokens(word, text, token = "regex", pattern = reg) %>%
  filter(!word %in% stop_words$word,  #stop_words fichier de mots compris dans tidytext
         str_detect(word, "[a-z]"))
#"My economic policy speech will be carried live at 12:15 P.M. Enjoy!"
# test<-tweet_words[1,]
# test2<-unnest_tokens(test,word, text, token = "regex", pattern = reg)
#petit détour stop_words compris dans tidytext - peut se traduire avec library(stopwords)

stop_french <- data.frame(word = stopwords::stopwords("fr"), stringsAsFactors = FALSE)
stop_french2 <- get_stopwords("fr","snowball")

#un barplot des mots les plus utilisés
words<-tweet_words %>% 
  count(word) %>% 
  arrange(desc(n))

wordsplus<-filter(words,n>=40)
ggplot(wordsplus , aes(x=factor(word, levels = word[order(n)]),y=n))+
  geom_bar(stat="identity",fill = "darkred")+
  labs(x = "words", y = "Number of occurence")+
  coord_flip()

#un nuage de mots parce que c'est joli ...???
#word clouds are a nice way of visualizing the frequency of words. lets compare the word clouds 
#from the android and iphone tweets. First we look at Android
library(wordcloud2)
wordsAndroid <- filter(tweet_words,source=="Android" & word!="https") %>% 
  count(word) %>% 
  filter(n>5)
wordsIPhone <- filter(tweet_words,source=="iPhone"& word!="https") %>% 
  count(word) %>% 
  filter(n>5)  

wordcloud2(wordsAndroid,size=1,)
wordcloud2(wordsIPhone,size=1)



# These should look familiar for anyone who has seen the feed. 
#Now let’s consider which words are most common 
# from the Android relative to the iPhone, and vice versa. 
#We’ll use the simple measure of log odds ratio, 
# calculated for each word as: 

android_iphone_ratios <- tweet_words %>%
  count(word, source) %>%
  filter(sum(n) >= 5) %>%
  spread(source, n, fill = 0) %>%
  ungroup() %>%
  mutate_each(funs((. + 1) / sum(. + 1)), -word) %>%
  mutate(logratio = log2(Android / iPhone)) %>%
  arrange(desc(logratio))

android_iphone_ratiosTop15<-android_iphone_ratios %>%
  group_by(logratio > 0) %>%
  top_n(15, abs(logratio)) %>% #get 15 biggest differnt words (comprenant des ex-aequo)
  ungroup() %>%
  mutate(word = reorder(word, logratio))

ggplot(android_iphone_ratiosTop15,
       aes(word, logratio, fill = logratio < 0)) + #create plot
  geom_bar(stat = "identity") + #add bar graph
  coord_flip() + #we want bars going horizontally, not vertically
  ylab("Android / iPhone log ratio") + #label axes
  scale_fill_manual(name = "", labels = c("Android", "iPhone"),values = c("red", "lightblue")) #color legend

# A few observations:
# 1-Most hashtags come from the iPhone. Indeed, almost no tweets from Trump’s Android contained hashtags, 
# with some rare exceptions like this one. (This is true only because we filtered out the quoted “retweets”, 
# as Trump does sometimes quote tweets like this that contain hashtags).
# 2-Words like “join” and “tomorrow”, and times like “7pm”, also came only from the iPhone. 
# The iPhone is clearly responsible for event announcements like this one (“Join me in Houston, Texas tomorrow night at 7pm!”)
# 3- A lot of “emotionally charged” words, like “badly”, “crazy”, “weak”, and “dumb”, were overwhelmingly 
# more common on Android. This supports the original hypothesis that this is the “angrier” or more hyperbolic account.


#Analyse des sentiments
#(Sentiment analysis: Trump’s tweets are much more negative than his campaign’s

# Since we’ve observed a difference in sentiment between the Android and iPhone tweets, 
# let’s try quantifying it. We’ll work with the NRC Word-Emotion Association lexicon, available from the tidytext 
# package, which associates words with 10 sentiments: positive, negative, anger, anticipation, disgust, fear, joy, 
#sadness,  surprise, and trust.

#Autre source https://www.tidytextmining.com/sentiment.html

library(tidytext)
library(lexicon)
library(textdata)

lexicon <- hash_sentiment_jockers #select our sentiemtn dicitonary


lexicon<-get_sentiments("nrc") #nécesite textdata
table(lexicon$sentiment)


sources <- tweet_words %>%
  group_by(source) %>%
  mutate(total_words = n()) %>%
  ungroup() %>%
  distinct(id, source, total_words)

by_source_sentiment <- tweet_words %>%
  inner_join(lexicon, by = "word") %>%
  count(sentiment, id) %>%
  ungroup() %>%
  complete(sentiment, id, fill = list(n = 0)) %>%
  inner_join(sources) %>%
  group_by(source, sentiment, total_words) %>%
  summarize(words = sum(n)) %>%
  ungroup()

head(by_source_sentiment)

#Test des différences 
library(broom)

sentiment_differences <- by_source_sentiment %>%
  group_by(sentiment) %>%
    do(tidy(poisson.test(.$words, .$total_words))) #comme source n'est pas spécifié... par défaut

sentiment_differences

